@@ -8,6 +8,7 @@ use Web::Solid::Auth::Agent;
 use Web::Solid::Auth::Util;
 use HTTP::Date;
 use File::LibMagic;
+use File::Basename;
 use MIME::Base64;
 use JSON;
 use Path::Tiny;
@@ -23,7 +24,9 @@ my $opt_recursive = undef;
 my $opt_skip      = undef;
 my $opt_real      = undef;
 my $opt_keep      = undef;
+my $opt_delete    = undef;
 my $opt_log       = 'log4perl.conf';
+my $opt_header    = [];
 
 GetOptions(
     "clientid|c=s" => \$clientid ,
@@ -31,25 +34,27 @@ GetOptions(
     "base|b=s"     => \$webbase ,
     "skip"         => \$opt_skip ,
     "keep"         => \$opt_keep ,
+    "delete"       => \$opt_delete ,
     "r"            => \$opt_recursive ,
     "x"            => \$opt_real ,
+    "H=s@"         => \$opt_header ,
     "log=s"        => \$opt_log ,
 );
 
 my $cmd = shift;
 
-unless ($webid)  {
-    print STDERR "Need a webid or SOLID_WEBID environment variable\n\n";
-    usage();
-}
-
 if (-e $opt_log) {
     Log::Log4perl::init($opt_log);
 }
 
-my $auth = Web::Solid::Auth->new(webid => $webid);
+my $auth = Web::Solid::Auth->new(webid => $webid, client_id => $clientid);
 my $agent = Web::Solid::Auth::Agent->new(auth => $auth);
 
+if ($webbase) {
+    # Remove the trailing slash
+    $webbase =~ s{\/$}{};
+}
+
 my $ret;
 
 if (0) {}
@@ -113,8 +118,8 @@ Usage
 usage: $0 [options] authenticate
 
 # Curl like interaction
-usage: $0 [options] headers METHOD URL
-usage: $0 [options] curl <...>
+usage: $0 [options] headers method url
+usage: $0 [options] curl -- [curl-opts] url
 
 # Interpret LDP responses
 usage: $0 [options] list /path/ | url        # folder listing
@@ -140,10 +145,11 @@ options:
     --clientid|c clientid    - optional the client-id
     --base|b base            - optional the base url for all requests
     --skip                   - skip files that already exist (mirror)
+    --delete                 - delete local files that are not at the remote location (mirror)
     --keep                   - keep containers (clean)
     -r                       - recursive (mirror, upload, clean)
     -x                       - do it for real (upload, clean)
-
+    -H "Header"              - add a header (get,post,put,head,delete)
 EOF
     exit 1
 }
@@ -193,11 +199,14 @@ sub _cmd_list {
     my $sparql =<<EOF;
 prefix ldp: <http://www.w3.org/ns/ldp#> 
 
-SELECT ?folder ?type {
-    ?folder a ?type .
-    FILTER (?type IN (ldp:BasicContainer, 
-                      ldp:Container,
+SELECT ?resource ?type {
+    ?resource a ?type .
+    FILTER (?type IN (
                       ldp:Resource,
+                      ldp:RDFSource,
+                      ldp:Container,
+                      ldp:BasicContainer, 
+                      ldp:IndirectContainer,
                       ldp:NonRDFSource
                       ) 
             )
@@ -208,7 +217,8 @@ EOF
 
     $util->sparql($model, $sparql, sub {
         my $res = shift;
-        my $name = $res->value('folder')->as_string; 
+        my $name = $res->value('resource')->as_string; 
+        $name = substr($name,length($webbase));
         $name =~ s/^\///; 
         my $type = $res->value('type')->as_string;
 
@@ -226,7 +236,9 @@ EOF
 sub cmd_get {
     my ($url) = @_; 
 
-    my $response = _cmd_get($url);
+    my %headers = _make_headers();
+
+    my $response = _cmd_get($url,%headers);
 
     return $response if $response && ref($response) eq '';
 
@@ -265,8 +277,9 @@ sub cmd_head {
     }
 
     my $iri = _make_url($url);
+    my %headers = _make_headers();
 
-    my $response = $agent->head($iri);
+    my $response = $agent->head($iri,%headers);
 
     unless ($response->is_success) {
         printf STDERR "%s - failed to $url\n" , $response->code;
@@ -290,8 +303,9 @@ sub cmd_options {
     }
 
     my $iri = _make_url($url);
+    my %headers = _make_headers();
 
-    my $response = $agent->options($iri);
+    my $response = $agent->options($iri,%headers);
 
     unless ($response->is_success) {
         printf STDERR "%s - failed to $url\n" , $response->code;
@@ -332,14 +346,16 @@ sub cmd_put {
     }
 
     my $iri = _make_url($url);
+    my %headers = _make_headers();
+    $headers{'Content-Type'} = $mimeType unless $headers{'Content-Type'};
 
     my $response;
 
     if ($file) {
-        $response = $agent->put($iri, $data, 'Content-Type' => $mimeType);
+        $response = $agent->put($iri, $data, %headers);
     }
     else {
-        $response = $agent->put($iri);
+        $response = $agent->put($iri,undef,%headers);
     }
 
     unless ($response->is_success) {
@@ -366,8 +382,10 @@ sub cmd_post {
     my $data = path($file)->slurp_raw;
 
     my $iri = _make_url($url);
+    my %headers = _make_headers();
+    $headers{'Content-Type'} = $mimeType unless $headers{'Content-Type'}; 
 
-    my $response = $agent->post($iri, $data, 'Content-Type' => $mimeType);
+    my $response = $agent->post($iri, $data, %headers);
 
     unless ($response->is_success) {
         printf STDERR "%s - failed to $url\n" , $response->code;
@@ -391,8 +409,9 @@ sub cmd_delete {
     }
 
     my $iri = _make_url($url);
+    my %headers = _make_headers();
 
-    my $response = $agent->delete($iri);
+    my $response = $agent->delete($iri, %headers);
 
     unless ($response->is_success) {
         printf STDERR "%s - failed to $url\n" , $response->code;
@@ -443,6 +462,39 @@ sub cmd_mirror {
             _cmd_mirror($file,$directory);
         }
     }
+
+    _cmd_mirror_delete($url,$files,$directory) if $opt_delete;
+}
+
+sub _cmd_mirror_delete {
+    my ($base,$files,$directory) = @_;
+    my $path_names;
+    for my $file (sort keys %$files) {
+        my $path = substr($file,length($base));
+        $path =~ s{^\/}{};
+        $path =~ s{\/$}{};
+        next unless length($path);
+        $path_names->{$path} = 1;
+    }
+
+    for my $path (glob("$directory/*")) {
+        my $basename = basename($path);
+        next if -d $path; # we always keep directories
+        next if $path =~ /^[\.~]/;
+
+        if ($path_names->{$basename}) {
+            # ok , known path
+        }
+        else {
+            if ($opt_real) {
+                print STDERR "deleting: $path\n";
+                unlink $path;
+            }
+            else {
+                print STDERR "deleting: $path [test : use -x for real delete]\n";
+            }
+        }
+    }
 }
 
 sub _cmd_mirror {
@@ -451,8 +503,6 @@ sub _cmd_mirror {
     my $path = $url;
     $path =~ s{.*\/}{};
 
-    print "$url -> $directory/$path\n";
-
     my %headers = ();
 
     if ($opt_skip && -e "$directory/$path" ) {
@@ -465,11 +515,17 @@ sub _cmd_mirror {
         $headers{'If-Modified-Since'} = HTTP::Date::time2str($mtime);
     }
 
-    my $response = _cmd_get($url,%headers);
+    if ($opt_real) {
+        print "$url -> $directory/$path\n";
+        my $response = _cmd_get($url,%headers);
 
-    return $response unless $response && ref($response) ne '';
+        return $response unless $response && ref($response) ne '';
 
-    path("$directory/$path")->spew_raw($response->decoded_content);
+        path("$directory/$path")->spew_raw($response->decoded_content);
+    }
+    else {
+        print "$url -> $directory/$path [test : use -x for real mirror]\n";
+    }
 
     return 0;
 }
@@ -587,6 +643,12 @@ sub cmd_clean {
 }
 
 sub cmd_authenticate {
+
+    unless ($webid) {
+        print STDERR "Need a WebId or SOLID_WEBID environment variable\n";
+        return 2;
+    }
+
     $auth->make_clean;
 
     my $auth_url = $auth->make_authorization_request;
@@ -620,6 +682,8 @@ sub cmd_curl {
     my $method = 'GET';
     my $url = $rest[-1];
 
+    shift @rest if @rest[0] eq '--';
+
     if (@rest) {
         for (my $i = 0 ; $i < @rest ; $i++) {
             if ($rest[$i] eq '-X') {
@@ -637,6 +701,11 @@ sub cmd_curl {
 sub cmd_access_token {
     my $access = $auth->get_access_token;
 
+    unless ($webid) {
+        print STDERR "Need a WebId or SOLID_WEBID environment variable\n";
+        return 2;
+    }
+    
     unless ($access && $access->{access_token}) {
         print STDERR "No access_token found. You are not logged in yet?\n";
         return 2;
@@ -690,9 +759,20 @@ sub cmd_id_token {
     return 0;
 }
 
+sub _make_headers {
+    my %headers = ();
+    for my $h (@$opt_header) {
+        my ($n,$v) = split(/\s*:\s*/,$h,2);
+        $headers{$n} = $v;
+    }
+    return %headers;
+}
+
 sub _make_url {
     my $url = shift;
 
+    return $url if $url =~ /^http.*/;
+
     return $url unless defined($webbase);
 
     return $url unless $url =~ /^\.?(\/.*)?/;
@@ -709,7 +789,6 @@ sub _headers {
 
     unless ($headers) {
         print STDERR "No access tokens found for $webid. Maybe you need to authenticate first?\n";
-        exit 2;
     }
 
     my @headers = ();
@@ -755,31 +834,31 @@ solid_auth.pl - A Solid management tool
 =head1 SYNOPSIS
 
       # Set your default webid
-      export SOLID_WEBID=https://hochstenbach.inrupt.net/profile/card#me
+      export SOLID_WEBID=https://timbl.inrupt.net/profile/card#me
 
       # Authentication to a pod
       solid_auth.pl authenticate
 
       # Get the http headers for a authenticated request
-      solid_auth.pl headers GET https://hochstenbach.inrupt.net/inbox/
+      solid_auth.pl headers GET https://timbl.inrupt.net/inbox/
 
       # Act like a curl command and fetch authenticated content
-      solid_auth.pl curl -X GET https://hochstenbach.inrupt.net/inbox/
+      solid_auth.pl curl -- -X GET https://timbl.inrupt.net/inbox/
 
       # Add some data
-      solid_auth.pl curl -X POST \
+      solid_auth.pl curl -- -X POST \
             -H "Content-Type: text/plain" \
             -d "abc" \
-            https://hochstenbach.inrupt.net/public/
+            https://timbl.inrupt.net/public/
     
       # Add a file
-      solid_auth.pl curl -X PUT \
+      solid_auth.pl curl -- -X PUT \
             -H "Content-Type: application/ld+json" \
             -d "@myfile.jsonld" \
-            https://hochstenbach.inrupt.net/public/myfile.jsonld 
+            https://timbl.inrupt.net/public/myfile.jsonld 
 
       # Set a solid base url
-      export SOLID_REMOTE_BASE=https://hochstenbach.inrupt.net
+      export SOLID_REMOTE_BASE=https://timbl.inrupt.net
 
       # List all resources on some Pod path
       solid_auth.pl list /public/
@@ -857,6 +936,10 @@ webserver.
 
 Skip resources that already exist (mirror).
 
+=item --delete
+
+Delete local files that are not in the remote container (mirror).
+
 =item --keep
 
 Keep containers when cleaning data (clean).
@@ -869,6 +952,90 @@ Recursive (clean, mirror, upload).
 
 Do it for real. The commands C<clean> and C<upload> will run by default in safe mode.
 
+=item -H name=value
+
+Add a header to a request (repeatable) for C<get>, C<post>, C<head>, C<options> and C<delete>.
+
+=back
+
+=head1 COMMAND
+
+=over 
+
+=item authenticate
+
+Start an authentication process for your WebId. You will be presented with a 
+URL that you need to open in a webbrowser. After a successfull login the 
+command can be closed. 
+
+The webbrowser needs to be opened on the same host as the where you where you
+run the solid_auth.pl command. 
+
+=item headers METHOD URL
+
+Return the Authentication and DPoP headers for a HTTP C<METHOD> request to C<URL>.
+
+=item curl CURL-OPTS
+
+Execute a curl command with Authentication and DPoP headers added. Add a C<--> 
+option to the C<CURL-OPTS> to stop solid_auth.pl from interpreting Curl options.
+
+=item list URL
+
+List the resources in a LDP container at URL.
+
+=item mirror [-rx] [--skip] [--delete] URL DIRECTORY
+
+Mirror the contents of a container to a local directory. Optional provide C<-r>
+option for recursive mirror.
+
+=item upload [-rx] DIRECTORY URL
+
+Upload a directorty to a container. Optional provide C<-r> option for recursive
+upload. With the C<-x> option, the command will run in demo mode.
+
+=item clean [-rx] [--keep] URL
+
+Clean all resources in a directory. Optional provide C<-r> option for recursive
+clean. With the C<-x> option, the command will run in demo mode. With the
+C<--keep> options all container will be kept.
+
+=item get URL
+
+Return the response of a HTTP GET request to URL.
+
+=item post URL FILE [MIMETYPE]
+
+Return the HTTP Message of a HTTP POST request of the FILE with MIMETYPE.
+Uses libmagic to guess the mimetype.
+
+=item put URL [FILE] [MIMETYPE]
+
+Return the HTTP Message of a HTTP PUT request of the FILE with MIMETYPE.
+Uses libmagic to guess the mimetype.
+
+When the URL ends with a slash (/), then a new container will be created.
+
+=item head URL
+
+Return the HTTP Header of a HTTP HEAD request to URL.
+
+=item head OPTIONS
+
+Return the HTTP Header of a HTTP OPTIONS request to URL.
+
+=item delete URL
+
+Return the HTTP Message of a HTTP DELETE request to URL.
+
+=item id_token
+
+Show the contents of the JWT id token.
+
+=item access_token
+
+Show the contents of the JWT access token.
+
 =back
 
 =head1 INSPIRATION
